function advice(d)
%ADVICE gives advice and suggestions on properties of IDDATA objects. 
%
%   ADVICE(DATA)

%   Copyright 1986-2003 The MathWorks, Inc.
%   $Revision: 1.11.4.2 $  $Date: 2004/04/10 23:15:41 $

% First check if the user is aware of certain possibilities
adv = {' '};
ny = size(d,'ny');
nu = size(d,'nu');
if ny==0
    adv ={'There is no output signal in this data set. Models cannot be estimated'...
            'but input spectra can be calculated by ETFE, SPA, and SPAFDR.'...
            ' '};
end
if nu==0
    dom = pvget(d,'Domain'); dom = lower(dom(1));
    if dom=='t'
    adv ={'There is no input signal in this data set. Parametric models of spectra'...
        'can be built using ARX, ARMAX, N4SID and PEM. Nonparametric models can be'...
        'estimated using SPA, ETFE and SPAFDR.'...
            ' '};
else
     adv ={'There is no input signal in this data set. For frequency domain data,'...
        'parametric models of spectra can be only be built using ARX.',...
        'Nonparametric models can be estimated using SPA, ETFE and SPAFDR.'...
            ' '};
end
end
if nu >0
    inter = unique(d.InterSample(:));
    if strcmp(inter{1},'zoh')
        adv = [adv,{...
                    'All your inputs have been denoted as ''zero order hold'' (''zoh''), i.e. they are'...
                    'assumed to be piecewise constant over the sampling interval.'...
                    'If the input is a sampled continuous signal and you plan to build or convert to'...
                    'time continuous models, it is recommended to mark the Intersample property as'...
                    '''First order hold'': Data.int = ''foh'' or Data.int = {''foh'',''foh'', ...} for'...
                    'multiinput signals. You may mix ''zoh'' with ''foh'' for the different inputs in '...
                    'the latter case.'...
                    ' '}];
    end
end
if isnan(d)
    adv = [adv,{...
                'Your data contains missing data points (marked as NaN). You must run MISDATA',...
                'before you use the estimation routines.'...
                ' '}];
end
u=d.InputData;
y=d.OutputData;
dom = d.Domain; dom = lower(dom(1));
if dom=='t';
    for kexp=1:length(u)
        if length(u)==1
            txtkexp = '';
        else
            txtkexp = [' in experiment number ',int2str(kexp)];
        end
        
        uu=u{kexp};
        nu = size(uu,2);
        
        yy=y{kexp};
        my = 0;
        for ky =1:ny
            my(ky)=mean(yy(:,ky));
        end
        for ku=1:nu
            if nu == 1
                txtku = ['The input'];
            else
                txtku= ['Input number ',int2str(ku)];
            end
            du = diff(uu(:,ku));
            ns = find(abs(du)>0.0001*max(uu(:,ku)));
            mu(ku) = mean(uu(:,ku));
            if isempty(ns)
                if length(u)==1
                    adv = [adv,{...
                                [txtku,txtkexp,' is essentially a constant. Estimating parameters',...
                                    ' associated with this'],...
                                ['input will be difficult.']....
                                ['If the input is a step occurring at time 0, then shift the data,'],...
                                ['and prepend zeros, to let the step happen at sample 10 or so.'],' '}];
                else
                    adv = [adv,{...
                                [txtku,txtkexp,' is essentially',...
                                    '  a constant.']...
                                'Estimating parameters associated with this input will be difficult'...
                                'unless the other experiments support this input.'...
                                ' '}];
                    
                end
            elseif length(ns)==1&ns<25
                adv = [adv,{...
                            [txtku,txtkexp,' is essentially a step',...
                                ' occuring at sample number ',int2str(ns),'.']...
                            'Estimating models of orders larger than this number will not take advantage'...
                            'of the information in this input.'...
                            'If models of higher order are of interest, it is recommended to use more steps,'...
                            'or to let the step occur later or to prepend the input output data with signals'...
                            'of constant levels, corresponding to a system equilibrium.'...
                            ' '}];
            elseif max(ns)<25
                adv = [adv,{...
                            [txtku,txtkexp,' has all its variation',...
                                ' occuring before sample number ',int2str(max(ns)),'.']...
                            'Estimating models of orders larger than this number will not take advantage'...
                            'of the information in this input.'...
                            'If models of higher order are of interest, it is recommended to use more variation,'...
                            'or to let the variation occur later or to prepend the input output data with signals'...
                            'of constant levels, corresponding to a system equilibrium.'...
                            ' '}];
            end
        end
    end
    if any(my>0.0001/max(my))|any(mu>0.001/max(mu))
        adv = [adv,{...
                    'All inputs and outputs are not zero mean. It it generally recommended to'...
                    'remove the means by DAT = DETREND(DAT), except in the following two cases:'...
                    '1. The signals are measured relative to a level that corresponds to a'...
                    '   physical equilibrium. This could e.g. be the case if step responses'...
                    '   are recorded from an equlibrium point.'...
                    '2. There is an integrator in the system, and the input and output '...
                    '   levels are essential to describe the effect of the integration.'...
                    ' '}];
    end
end
if ny>0&nu>0
    was = warning;
    warning off
    [fbck,fbck0,nudir] = feedback(d);
    warning(was)
    if any(nudir)
        
        adv = [adv,{...
                    'There is an indication that the system has a direct response from'...
                    ['input number(s) ',int2str(nudir),' at time t to y(t).'],...
                    'There may be two reasons for this:'...
                    ' 1. There is direct feedback from y(t) to u(t) (like a P-regulator).'...
                    '    In that case it is essential not to let this feedback influence the model.'...
                    '    It is thus important always to use nk>0 for these inputs in state-space and'...
                    '    input-output models.'...
                    ' 2. The system has a direct term (relative degree zero). Then it is essential'...
                    '    to use nk = 0 for these inputs in all models. (Note that state-space models'...
                    '    have nk = 1 as default, so use'...
                    '    MOD = PEM(Data,n,''nk'',0), where n is the model order.)'...
                    ' '}];
    end
    % This could be done as i/o pairs:
    
    fblev = max(max(fbck)');
    if ~isnan(fblev)
        if fblev < 60
            adv = [adv,{...
                        'There is no significant indication of feedback in the data.'...
                    }];
            if any(nudir)
                adv = [adv,{...
                            '(Unless you decide that the direct term is due to feedback.)'...
                        }];
            end
            adv = [adv,{' '}];
        else
            if fblev>99
                
                txt = 'a very strong indication';
            elseif fblev>90
                txt = 'a strong indication';
                
            else
                txt = 'a possible indication';
            end
            adv = [adv,{...
                        ['There is ',txt,' of feedback in the data.'],...
                        ' You should be careful when interpreting the results of SPA and also interpret'...
                        ' the results of output error models with care.'...
                        '(Output error models result from the OE command or setting ''DisturbanceModel''= ''None'''...
                        'in state-space models.)'...
                        ' '}];
        end
    end
end % if ny>0
% PE
if nu>0
    [pe,maxnr] = pexcit(d);
    pemu = find(pe==maxnr-1);
    notpemu = setdiff([1:nu],pemu);
    if ~isempty(pemu)&all(pemu == [1:nu])
        if nu == 1
            txt = 'The input is';
        else
            txt = 'All inputs are';
        end
        adv =[adv,{...
                    [txt,' persistently exciting of high order.']...
                    'This means that you should be able to build models of orders up to '...
                    [int2str(maxnr),' or so, if necessary.']...
                    ' '}];
    else
        if nu ==1
            adv = [adv,{...
                        ['The input is persistently exciting if order ',int2str(pe(1)),'. This means that you']...
                        'will encounter problems if estimating models of order higher than this.'...
                        ' '}];
        else
            [mpr,mu]=min(pe);
            adv = [adv,{...
                        ['Input number ',int2str(mu),' is persistently exciting of order ',int2str(mpr),'.'],...
                        ' This means that you will have problems when estimation models of order'...
                        ' higher than this, at least for model parameters associated with this input.'...
                        [' The excitation orders for all the inputs are [',int2str(pe),'].'...
                            ' ']}];
        end
    end
end                


% Find the right level of this

disp(char(adv));
