function [Props,AsgnVals] = pnames(dat)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(DAT,'true')  returns the list PROPS of
%   public properties of the object DAT(a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector 
%   of strings).  PROPS contains the true case-sensitive property names.
%
%   [PROPS,ASGNVALS] = PNAMES(SYS,'lower')  returns lowercase property
%   names.  This helps speed up name matching in GET and SET.
%
%   See also  GET, SET.

 %       Copyright 1986-2001 The MathWorks, Inc.
%       $Revision: 1.7 $  $Date: 2001/04/06 14:22:02 $

% DAT-specific public properties
    Props = {'Domain' ;'Name'; 'OutputData' ; 'OutputName';...
         'OutputUnit';'InputData';'InputName';'InputUnit';...
         'Period';'InterSample';'Ts' ; 'Tstart';...
         'SamplingInstants';'TimeUnit';...
         'ExperimentName';'Notes';'UserData';'Version';'Utility'};
 
% Construct outputs
if nargout<=1,
   % Only property names needed: 
else
   % Also return values
   AsgnVals = {'''Time''/''Frequency''';...
               'String';...
               'N-by-Ny matrix or 1-by-Ne cell array '; ...
               'Ny-by-1 cell array of strings';...
               'Ny-by-1 cell array of strings';...
               'N-by-Nu matrix or 1-by-Ne cell array'; ...
               'Nu-by-1 cell array of strings';...
               'Nu-by-1 cell array of strings';...
               'Nu-by-1 matrix of input periods (inf: non-periodic)';...
               'Nu-by-1 cell array: ''zoh''/''foh''/''bl''';...
               'Scalar  (Sample Interval, empty if non-equal sampling)';...
               'Scalar  (Starting time)';...
               'N-by-1 matrix (leave empty if equidistant sampling)';...
               'String';
               '1-by-Ne cell array (Ne = No of experiments)';...
               'Cell array of strings';...
               'Arbitrary';...
               'Internal Use';...
               'Internal Use'};


      
 
end

% end iddata/pnames.m
