function [Props,AsgnVals] = pnames(sys,flag)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(DAT,'true')  returns the list PROPS of
%   public properties of the object DAT(a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector 
%   of strings).  PROPS contains the true case-sensitive property names.
%
%   [PROPS,ASGNVALS] = PNAMES(SYS,'lower')  returns lowercase property
%   names.  This helps speed up name matching in GET and SET.
%
%   See also  GET, SET.

%       Author(s): P. Gahinet, 7-8-97
%       Copyright 1986-2001 The MathWorks, Inc.
%       $Revision: 1.9 $  $Date: 2001/04/06 14:22:06 $
no = nargout;

% SS-specific public properties and their assignable values
  Props = {'Name';'Frequency';'ResponseData';...
       'SpectrumData';'CovarianceData';'NoiseCovariance';'Units';'Ts';'InputDelay';...
       'EstimationInfo';'InputName';'OutputName';'InputUnit';'OutputUnit';'Notes';...
       'UserData';'Version';'Utility'};
if no>1,
   AsgnVals = {'string';'vector of frequency points (Nf frequency points)';...
         'Ny-by-Nu-by-Nf array of complex reponses';...
          'Ny-by-Ny-by-Nf array of noise spectrum';...
          'Ny-by-Nu-by-Nf-by-2-by-2 array. cd(ky,ku,kf,:,:) gives the cvm';...
           'Ny-by-Ny-by-Nf-by-4 array. cd(ky,ku,kf,:,:) gives the cvm';...
                '[''rad/s''|''Hz'']';...
                'scalar';...
                'Nu-by-1 vector';...
                'structure';...
                'Nu-by-1 cell array of strings';...
                 'Ny-by-1 cell array of strings';...
 'Nu-by-1 cell array of strings';...
 'Ny-by-1 cell array of strings';...
 'Array or cell array of strings';...
                'Arbitrary';...
               'Internal Use';...
               'Internal Use'};
end

% Add public parent properties unless otherwise requested
%if nargin==1,
%   if no==1,
%      Props = [Props ; pnames(sys.lti)];
%   else
%      [LTIprops,LTIvals] = pnames(sys.lti);
%      Props = [Props ; LTIprops];
%      AsgnVals = [AsgnVals ; LTIvals];
%   end
%end

