function [Props,AsgnVals] = pnames(m, flag)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(M)  returns the list PROPS of
%   public properties of the object M (a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector
%   of strings).  PROPS contains the true case-sensitive property names.
%   These include the public properties of M's parent(s).
%
%   See also  GET, SET.

%       Copyright 1986-2001 The MathWorks, Inc.
%       $Revision: 1.5 $  $Date: 2001/08/03 14:24:34 $


% IDMODEL properties
Props = {'A';'B';'C';'D';'K';'X0';...
         'dA';'dB';'dC';'dD';'dK';'dX0';'MfileName';'StateName';...
'CDmfile';'FileArgument';'InitialState';'DisturbanceModel'};

% Add public parent properties unless otherwise requested
% Also return assignable values if needed

if nargin==1
  if nargout==1
     Props = [Props; pnames(m.idmodel)];
  else
     AsgnVals = {'A-matrix (cannot be set)';...
           'B-matrix (cannot be set)';...
           'C-matrix (cannot be set';...
           'D-matrix (cannot be set)';...
           'K-matrix (cannot be set)';...
           'Vector of initial states (cannot be set)';...
                       'Std deviation of A-matrix (cannot be set)';...
           'Std deviation of B-matrix (cannot be set)';...
'Std deviation of C-matrix (cannot be set)';...
'Std deviation of D-matrix (cannot be set)';...
'Std deviation of K-matrix (cannot be set)';...
'Std deviation of X0-matrix (cannot be set)';...
           'Name of Mfile defining the structure (string)';...
            		 'Nx-by-1 cell array of strings'; ...
            'Mfile''s handling of cont/discr ([''cd'',''c'',''d''])';...
           'Optional argument to be passed on to mfile (arbitrary)';...
           '[''Model''|''Auto''|''Estimate''|''Zero''|''Backcast''|''Fixed'']';...
           '[''Model''|''Estimate''|''None''|''Fixed'']'...		 
     };
    [IDMProps, IDMVals] = pnames(m.idmodel);
     Props = [Props ; IDMProps];
     AsgnVals = [AsgnVals ; IDMVals]; 
  end
end
