function [Props,AsgnVals] = pnames(m, flag)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(M)  returns the list PROPS of
%   public properties of the object M (a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector
%   of strings).  PROPS contains the true case-sensitive property names.
%   These include the public properties of M's parent(s).
%
%   See also  GET, SET.

%       Author(s): P. Gahinet, 7-8-97
%       Copyright 1986-2001 The MathWorks, Inc.
%       $Revision: 1.6 $  $Date: 2001/04/06 14:22:19 $


% IDMODEL properties
Props = {'a';'b';'c';'d';'f';...
      'da';'db';'dc';'dd';'df';...
      'na' ; 'nb' ; 'nc' ; 'nd' ; 'nf' ; 'nk' ; 'InitialState'};

% Add public parent properties unless otherwise requested
% Also return assignable values if needed

if nargin==1
  if nargout==1
     Props = [Props; pnames(m.idmodel)];
  else
     AsgnVals = {'A-polynomial (row vector)';...
           'B-polynomial (matrix with row i for the i:th input)';...
           'C-polynomial (row vector)';...
           'D-polynomial (row vector)';...
           'F-polynomial (matrix with row i for the i:th input)';...
           'Std deviation of A-polynomial (cannot be set)';...
           'Std deviation of B-polynomial (cannot be set)';...
'Std deviation of C-polynomial (cannot be set)';...
'Std deviation of D-polynomial (cannot be set)';...
'Std deviation of F-polynomial (cannot be set)';... 
'Number of common poles (scalar)';...
           'Number of zeros+1 for the i:th input (vector)';...
           'Number of zeros in the noise model (scalar)';...
           'Number of poles in the noise model (scalar)';...
           'Number of poles for the i:th input (vector)';...
           'Number of delays for the i:th input (vector)';...
           '[ ''Auto'' | ''Zero'' | ''Estimate'' | ''Backcast'' ]'};
     [IDMProps, IDMVals] = pnames(m.idmodel);
     Props = [Props ; IDMProps];
     AsgnVals = [AsgnVals ; IDMVals]; 
  end
end
