function [Props,AsgnVals] = pnames(m, flag)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(M)  returns the list PROPS of
%   public properties of the object M (a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector
%   of strings).  PROPS contains the true case-sensitive property names.
%   These include the public properties of M's parent(s).
%
%   See also  GET, SET.

%       Author(s): P. Gahinet, 7-8-97
%       Copyright 1986-2001 The MathWorks, Inc.
%       $Revision: 1.7 $  $Date: 2001/04/06 14:22:22 $


% IDMODEL properties
Props = {'A';'B';'C';'D';'K';'X0' ;...
      'dA';'dB';'dC';'dD';'dK';'dX0';...
      'SSParameterization';...
      'As' ; 'Bs' ; 'Cs' ; 'Ds' ; 'Ks' ; 'X0s'; 'StateName'; 'InitialState';...
	 'nk';'DisturbanceModel';...
     'CanonicalIndices'};

% Add public parent properties unless otherwise requested
% Also return assignable values if needed

if nargin==1
  if nargout==1
     Props = [Props; pnames(m.idmodel)];
  else
     AsgnVals = {'A-matrix (Nx-by-Nx matrix)';...
           'B-matrix (Nx-by-Nu matrix)';...
           'C-matrix (Ny-by-Nx matrix)';...
           'D-matrix (Nu-by-Ny matrix)';...
           'K-matrix (Nx-by-Ny matrix)';...
           'X0-matrix (initial states) (Nx-by-1 matrix)';...
		 'Nx-by-1 cell array of strings'; ...
           'Std deviation of A-matrix (cannot be set)';...
           'Std deviation of B-matrix (cannot be set)';...
'Std deviation of C-matrix (cannot be set)';...
'Std deviation of D-matrix (cannot be set)';...
'Std deviation of K-matrix (cannot be set)';...
'Std deviation of X0-matrix (cannot be set)';...
           '[''Free''|''Canonical''|''Structured'']';...
           'Structure matrix for A (NaN denoting free parameter)';...
           'Structure matrix for B (NaN denoting free parameter)';...
           'Structure matrix for C (NaN denoting free parameter)';...
           'Structure matrix for D (NaN denoting free parameter)';...
           'Structure matrix for K (NaN denoting free parameter)';...
           'Structure matrix for X0 (NaN denoting free parameter)';...
           '[''Auto''|''Backcast''| ''Estimate'' | ''Zero'' | ''Fixed'']';... 
           'Model delays in response from u to y (nu-by-1 vector)';...
           '[ ''Estimate'' | ''None'' | ''Fixed'']';...
                     'Row vector or ''Auto'''};
  [IDMProps, IDMVals] = pnames(m.idmodel);
     Props = [Props ; IDMProps];
     AsgnVals = [AsgnVals ; IDMVals]; 
  end
end
